@@ -17,7 +17,6 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-
 #include <stdio.h>
 #include <unistd.h>
 #include <signal.h>
@@ -45,7 +44,7 @@
 #define SYSCONFFILE SYSCONFDIR "/" PACKAGE "/" CONFFILE
 #define DATACONFDIR DATADIR "/" PACKAGE "/"
 #define SYSCONFCONFDIR SYSCONFDIR "/" PACKAGE "/"
-#define PIDFILE PIDDIR "/" PACKAGE ".pid"
+#define PIDFILE HALEVT_PIDDIR "/" PACKAGE ".pid"
 
 static GMainLoop *loop;
 
@@ -55,7 +54,6 @@ void halevt_signal_handler(int sig)
     g_main_loop_quit (loop);
 }
 
-
 #define DIRECTORY_NR 3
 
 char **halevt_find_conf_files()
@@ -73,17 +71,13 @@ char **halevt_find_conf_files()
     int i;
     DIR *dir;
     struct dirent *next_file;
-    char *filename;
+    char *filename = NULL;
     char *extension;
     char *string_pos;
     char **current_basefile;
     int found;
 
-    if (conffiles == NULL || basefiles == NULL)
-    {
-        DEBUG(_("Out of memory"));
-        return NULL;
-    }
+    if (conffiles == NULL || basefiles == NULL) { goto oom; }
     basefiles[0] = NULL;
     conffiles[0] = NULL;
 
@@ -92,8 +86,7 @@ char **halevt_find_conf_files()
     {
         if ((homedir =  (char *) malloc ((1 + strlen("/.") + strlen (home) + strlen (PACKAGE) + strlen ("/")))) == NULL)
         {
-            DEBUG(_("Out of memory"));
-            return NULL;
+            goto oom;
         }
         strcpy (homedir, home);
         strcat (homedir, "/.");
@@ -112,17 +105,25 @@ char **halevt_find_conf_files()
         {
            if ((filename = (char *) malloc (strlen (directories[i]) + strlen (next_file->d_name) + 1)) == NULL)
            {
-               DEBUG(_("Out of memory"));
-               return NULL;
+               closedir(dir);
+               goto oom;
            }
            strcpy (filename, directories[i]);
            strcat (filename, next_file->d_name);
 
            if (stat (filename, &statbuf) == 0)
            {
-               if (! S_ISREG(statbuf.st_mode)) { continue ;}
+               if (! S_ISREG(statbuf.st_mode))
+               {
+                   free(filename);
+                   continue;
+               }
                /* special case, a file named .xml... */
-               if (! strcmp (CONFFILEEXTENSION, next_file->d_name)) { continue; }
+               if (! strcmp (CONFFILEEXTENSION, next_file->d_name))
+               {
+                   free(filename);
+                   continue;
+               }
 
                /* process a file only if it ends with .xml */
                string_pos = next_file->d_name;
@@ -135,34 +136,38 @@ char **halevt_find_conf_files()
                    }
                    string_pos += strlen (CONFFILEEXTENSION);
                }
-               if (extension == NULL) { continue ;}
+               if (extension == NULL)
+               {
+                   free(filename);
+                   continue ;
+               }
                found = 0;
-               current_basefile = basefiles;
-               while (*current_basefile != NULL)
+               WALK_NULL_ARRAY(current_basefile, basefiles)
                {
                    if (! strcmp (*current_basefile, next_file->d_name))
                    {
                        found = 1;
                        break;
                    }
-                   current_basefile++;
                }
                if (! found)
                {
-                   conffile_nr++;
-                   basefiles = (char **) realloc (basefiles, conffile_nr * sizeof(char *));
-                   conffiles =  (char **) realloc (conffiles, conffile_nr * sizeof(char *));
-                   if (basefiles == NULL || conffiles == NULL)
+                   char **tmp_basefiles = (char **) realloc (basefiles, ++conffile_nr * sizeof(char *));
+                   char **tmp_conffiles =  (char **) realloc (conffiles, conffile_nr * sizeof(char *));
+                   if (tmp_basefiles == NULL || tmp_conffiles == NULL)
                    {
-                       DEBUG(_("Out of memory"));
-                       return NULL;
+                       free(tmp_basefiles);
+                       free(tmp_conffiles);
+                       closedir(dir);
+                       goto oom;
                    }
-                   basefiles[conffile_nr - 2] = strdup (next_file->d_name);
+                   basefiles = tmp_basefiles;
+                   conffiles = tmp_conffiles;
                    conffiles[conffile_nr - 2] = filename;
                    if ((basefiles[conffile_nr - 2] = strdup (next_file->d_name)) == NULL)
                    {
-                       DEBUG(_("Out of memory"));
-                       return NULL;
+                       closedir(dir);
+                       goto oom;
                    }
                    conffiles[conffile_nr - 1] = NULL;
                    basefiles[conffile_nr - 1] = NULL;
@@ -173,27 +178,31 @@ char **halevt_find_conf_files()
                DEBUG(_("stat failed for %s: %s"), filename, strerror (errno));
            }
        }
+       closedir(dir);
     }
 
-    current_basefile = basefiles;
-    while (*current_basefile != NULL)
-    {
-       free (*current_basefile);
-       current_basefile++;
-    }
-    free (basefiles);
+    FREE_NULL_ARRAY(char *, basefiles, free);
+    free(homedir);
 
     return conffiles;
+
+oom:
+    DEBUG(_("Out of memory"));
+    FREE_NULL_ARRAY(char *, conffiles, free);
+    FREE_NULL_ARRAY(char *, basefiles, free);
+    free(homedir);
+    free(filename);
+    return NULL;
 }
 
-void halevt_clear_pidfile(char *file)
+void halevt_clear_pidfile(const char *file)
 {
     if (file != NULL)
     {
         DEBUG(_("Deleting lockfile %s"), file);
         if (unlink(file) == -1)
         {
-            DEBUG(_("Failed to delete lockfile %s!"), file);
+            DEBUG(_("Failed to delete lockfile %s"), file);
         }
     }
 }
@@ -213,6 +222,9 @@ void usage()
 ));
 }
 
+int halevt_fork = 0;
+int halevt_report = 0;
+
 /**
  * main function.
  */
@@ -220,9 +232,9 @@ int main(int argc, char *argv[])
 {
     char **conffiles = NULL;
     char **current_conffile;
-    char *user = NULL;
-    char *group = NULL;
-    char *pid_file = NULL;
+    const char *user = NULL;
+    const char *group = NULL;
+    const char *pid_file = NULL;
     struct group *group_struct;
     struct passwd *passwd_struct;
     struct stat statbuf;
@@ -231,9 +243,6 @@ int main(int argc, char *argv[])
     int c;
     int do_fork = 1;
 
-    halevt_fork = 0;
-    halevt_report = 0;
-
     setlocale (LC_ALL, "");
     bindtextdomain (PACKAGE, DATADIR "/locale/");
     textdomain (PACKAGE);
@@ -242,7 +251,7 @@ int main(int argc, char *argv[])
     {
         c = getopt(argc, argv, "ic:u:g:fp:h");
         if (c == -1) { break ; }
-        switch (c) 
+        switch (c)
         {
             case 'c':
                if ((conffiles = (char **) malloc (sizeof(char *) * 2)) == NULL)
@@ -255,8 +264,8 @@ int main(int argc, char *argv[])
                    DEBUG(_("Out of memory"));
                    exit (1);
                }
-               if  (stat (conffiles[0], &statbuf) < 0) 
-               { 
+               if  (stat (conffiles[0], &statbuf) < 0)
+               {
                    DEBUG(_("stat on %s failed: %s"), conffiles[0], strerror (errno));
                    exit (1);
                }
@@ -267,25 +276,13 @@ int main(int argc, char *argv[])
                do_fork = 0;
                break;
             case 'u':
-               if ((user = strdup(optarg)) == NULL)
-               {
-                   DEBUG(_("Out of memory"));
-                   exit (1);
-               }
+               user = optarg;
                break;
             case 'g':
-               if ((group = strdup(optarg)) == NULL)
-               {
-                   DEBUG(_("Out of memory"));
-                   exit (1);
-               }
+               group = optarg;
                break;
             case 'p':
-               if ((pid_file = strdup(optarg)) == NULL)
-               {
-                   DEBUG(_("Out of memory"));
-                   exit (1);
-               }
+               pid_file = optarg;
                break;
             case 'f':
                do_fork = 0;
@@ -301,8 +298,8 @@ int main(int argc, char *argv[])
     /* change id */
     if (geteuid() == 0)
     {
-        if (user == NULL) { user = USER; }
-        if (group == NULL) { group = GROUP; }
+        if (user == NULL) { user = HALEVT_DEFAULT_USER; }
+        if (group == NULL) { group = HALEVT_DEFAULT_GROUP; }
         if (pid_file == NULL) { pid_file = PIDFILE; }
         if (!strcmp(pid_file, "-")) { pid_file = NULL; }
 
@@ -336,25 +333,20 @@ int main(int argc, char *argv[])
     {
        /* parse conf file */
         if (conffiles == NULL) { conffiles = halevt_find_conf_files (); }
-        if (conffiles[0] == NULL)
+        if (conffiles == NULL || conffiles[0] == NULL)
         {
             DEBUG(_("No configuration file found"));
             exit (1);
         }
-        current_conffile = conffiles;
-        while (*current_conffile != NULL)
+        WALK_NULL_ARRAY(current_conffile, conffiles)
         {
             if (! halevt_parse_config (*current_conffile))
             {
                 DEBUG(_("Configuration file %s parsing failed"), *current_conffile);
                 exit(1);
             }
-            if (halevt_fork)
-            {
-                DEBUG(_("Using configuration file %s"), *current_conffile);
-            }
+            DEBUG(_("Using configuration file %s"), *current_conffile);
             free (*current_conffile);
-            current_conffile++;
         }
         free (conffiles);
     }
@@ -385,7 +377,7 @@ int main(int argc, char *argv[])
             {
                 DEBUG(_("Open pid file %s failed: %s"), pid_file, strerror(errno));
                 exit (1);
-            } 
+            }
             if (lockf(fileno(file), F_TLOCK, 0) != 0)
             {
                 DEBUG(_("Lock %s failed: %s"), pid_file, strerror(errno));
@@ -403,9 +395,9 @@ int main(int argc, char *argv[])
     halevt_run_oninit ();
 
     loop = g_main_loop_new(NULL, FALSE);
-    if (!loop) 
+    if (!loop)
     {
-        DEBUG(_("Error creating main loop!"));
+        DEBUG(_("Error creating main loop"));
         return 1;
     }
 
@@ -413,15 +405,20 @@ int main(int argc, char *argv[])
     signal(SIGINT, halevt_signal_handler);
     signal(SIGHUP, halevt_signal_handler);
 
-/*    
-    DEBUG(_("Entering main loop.")); 
+/*
+    DEBUG(_("Entering main loop."));
 */
     g_main_loop_run(loop);
-/*    
-    DEBUG(_("Exiting normally.")); 
+/*
+    DEBUG(_("Exiting normally."));
 */
 
     if (halevt_fork) { halevt_clear_pidfile(pid_file); }
 
+    g_main_loop_unref(loop);
+    halevt_cleanup_HAL();
+    halevt_free_config();
+    halevt_free_devices();
+
     return 0;
 }
