@@ -17,7 +17,6 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-
 #include <libxml/tree.h>
 #include <stdio.h>
 #include <string.h>
@@ -142,6 +141,14 @@ halevt_exec *halevt_new_exec(const xmlChar *exec)
    return result_exec;
 }
 
+static void halevt_free_exec(halevt_exec *exec)
+{
+   free(exec->string);
+   free(exec->parsed_string);
+   free(exec->elements);
+   free(exec);
+}
+
 /* for debugging */
 char *halevt_print_exec(halevt_exec *exec_str)
 {
@@ -173,29 +180,19 @@ char *halevt_print_exec(halevt_exec *exec_str)
    return result;
 }
 
-halevt_boolean_expression *halevt_new_match(const xmlChar *match)
-{
-   char *string;
-   if ((string = (char *) xmlStrdup(match)) == NULL)
-   {
-      return NULL;
-   }
-   return halevt_new_boolean_expression (string);
-}
-
 halevt_insertion *halevt_add_insertion(const xmlChar *match, const xmlChar *exec)
 {
    halevt_insertion *new_insertion;
    new_insertion = malloc(sizeof(halevt_insertion));
    if (new_insertion != NULL)
    {
-      if ((new_insertion->match = halevt_new_match(match)) == NULL)
+      if ((new_insertion->match = halevt_new_boolean_expression(match)) == NULL)
       {
          free(new_insertion);
          return NULL;
       }
       if ((new_insertion->exec = halevt_new_exec(exec)) == NULL)
-      { 
+      {
          halevt_free_boolean_expression (new_insertion->match);
          free(new_insertion);
          return NULL;
@@ -209,19 +206,26 @@ halevt_insertion *halevt_add_insertion(const xmlChar *match, const xmlChar *exec
    return new_insertion;
 }
 
+static void halevt_free_insertion(halevt_insertion *insertion)
+{
+   halevt_free_boolean_expression(insertion->match);
+   halevt_free_exec(insertion->exec);
+   free(insertion);
+}
+
 halevt_oninit *halevt_add_oninit(const xmlChar *match, const xmlChar *exec)
 {
    halevt_oninit *new_oninit;
    new_oninit = malloc(sizeof(halevt_oninit));
    if (new_oninit != NULL)
    {
-      if ((new_oninit->match = halevt_new_match (match)) == NULL)
+      if ((new_oninit->match = halevt_new_boolean_expression (match)) == NULL)
       {
          free(new_oninit);
          return NULL;
       }
       if ((new_oninit->exec = halevt_new_exec (exec)) == NULL)
-      { 
+      {
          halevt_free_boolean_expression (new_oninit->match);
          free(new_oninit);
          return NULL;
@@ -235,19 +239,26 @@ halevt_oninit *halevt_add_oninit(const xmlChar *match, const xmlChar *exec)
    return new_oninit;
 }
 
+static void halevt_free_oninit(halevt_oninit *oninit)
+{
+   halevt_free_boolean_expression(oninit->match);
+   halevt_free_exec(oninit->exec);
+   free(oninit);
+}
+
 halevt_removal *halevt_add_removal(const xmlChar *match, const xmlChar *exec)
 {
    halevt_removal *new_removal;
    new_removal = malloc(sizeof(halevt_removal));
    if (new_removal != NULL)
    {
-      if ((new_removal->match = halevt_new_match(match)) == NULL)
+      if ((new_removal->match = halevt_new_boolean_expression(match)) == NULL)
       {
          free (new_removal);
          return NULL;
       }
       if ((new_removal->exec = halevt_new_exec(exec)) == NULL)
-      { 
+      {
          halevt_free_boolean_expression (new_removal->match);
          free (new_removal);
          return NULL;
@@ -261,34 +272,41 @@ halevt_removal *halevt_add_removal(const xmlChar *match, const xmlChar *exec)
    return new_removal;
 }
 
-halevt_condition *halevt_add_condition(const xmlChar *match, 
+static void halevt_free_removal(halevt_removal *removal)
+{
+   halevt_free_boolean_expression(removal->match);
+   halevt_free_exec(removal->exec);
+   free(removal);
+}
+
+halevt_condition *halevt_add_condition(const xmlChar *match,
     const xmlChar *exec, const xmlChar *name, const xmlChar *value)
 {
    halevt_condition *new_condition;
    new_condition = malloc (sizeof(halevt_condition));
    if (new_condition != NULL)
    {
-      if ((new_condition->match = halevt_new_match(match)) == NULL)
+      if ((new_condition->match = halevt_new_boolean_expression(match)) == NULL)
       {
          free(new_condition);
          return NULL;
       }
       if ((new_condition->name = (char *) xmlStrdup(name)) == NULL)
-      { 
+      {
          halevt_free_boolean_expression (new_condition->match);
          free(new_condition);
          return NULL;
       }
       if (value == NULL) { new_condition->value = NULL; }
       else if ((new_condition->value = (char *) xmlStrdup(value)) == NULL)
-      { 
+      {
          halevt_free_boolean_expression (new_condition->match);
          free(new_condition->name);
          free(new_condition);
          return NULL;
       }
       if ((new_condition->exec = halevt_new_exec(exec)) == NULL)
-      { 
+      {
          halevt_free_boolean_expression (new_condition->match);
          free(new_condition->name);
          if (new_condition->value != NULL)
@@ -305,37 +323,41 @@ halevt_condition *halevt_add_condition(const xmlChar *match,
    return new_condition;
 }
 
-halevt_property *halevt_add_property(const xmlChar *match, const xmlChar *name)
+static void halevt_free_condition(halevt_condition *condition)
+{
+   halevt_free_boolean_expression(condition->match);
+   halevt_free_exec(condition->exec);
+   free(condition->name);
+   free(condition);
+}
+
+halevt_property *halevt_add_property(const xmlChar *match, xmlChar *name)
 {
    halevt_property *new_property;
-   char *property_name;
    new_property = malloc(sizeof(halevt_property));
    if (new_property != NULL)
    {
-      if ((new_property->name = (char *) xmlStrdup(name)) == NULL)
-      {
-         free(new_property);
-         return NULL;
-      }
-      property_name = halevt_hal_string (new_property->name);
-      if (property_name == NULL)
+      if ((new_property->name = halevt_hal_string(name)) == NULL)
       {
          DEBUG(_("Invalid Property name: %s"), name);
-         free(new_property->name);
          free(new_property);
          return NULL;
       }
-      if ((new_property->match = halevt_new_match(match)) == NULL)
+      if ((new_property->name = (char *)xmlStrdup(new_property->name)) == NULL)
+      {
+         free(new_property);
+         return NULL;
+      }
+      if ((new_property->match = halevt_new_boolean_expression(match)) == NULL)
       {
          free(new_property->name);
          free(new_property);
          return NULL;
       }
-      
+
 /*
       printf ("add_property %s, %s\n", name, match);
 */
-      new_property->name = property_name;
       new_property->next = halevt_property_root;
       new_property->action = NULL;
       halevt_property_root = new_property;
@@ -343,12 +365,12 @@ halevt_property *halevt_add_property(const xmlChar *match, const xmlChar *name)
    return new_property;
 }
 
-halevt_property_action *halevt_add_property_value(halevt_property *property, 
+halevt_property_action *halevt_add_property_value(halevt_property *property,
     const xmlChar *exec, const xmlChar *value)
 {
    halevt_property_action *new_property_action;
    new_property_action = malloc (sizeof(halevt_property_action));
-   
+
    if (new_property_action != NULL)
    {
       if ((new_property_action->value = (char *) xmlStrdup(value)) == NULL)
@@ -356,7 +378,7 @@ halevt_property_action *halevt_add_property_value(halevt_property *property,
          free (new_property_action);
          return NULL;
       }
-      if ((new_property_action->exec = halevt_new_exec(exec)) == NULL) 
+      if ((new_property_action->exec = halevt_new_exec(exec)) == NULL)
       {
          free (new_property_action->value);
          free (new_property_action);
@@ -371,12 +393,27 @@ halevt_property_action *halevt_add_property_value(halevt_property *property,
    return new_property_action;
 }
 
+static void halevt_free_property_value(halevt_property_action *action)
+{
+   halevt_free_exec(action->exec);
+   free(action->value);
+   free(action);
+}
+
+static void halevt_free_property(halevt_property *property)
+{
+   FREE_LINKED_LIST(halevt_property_action, property->action, halevt_free_property_value);
+   halevt_free_boolean_expression(property->match);
+   free(property->name);
+   free(property);
+}
+
 int halevt_parse_config (char const *path)
 {
    xmlDocPtr doc;
    xmlNodePtr cur;
    xmlNodePtr device;
-   char *root_node = "Configuration";   
+   char *root_node = "Configuration";
 
    doc = xmlParseFile (path);
    if (doc == NULL)
@@ -408,8 +445,7 @@ int halevt_parse_config (char const *path)
       }
       else
       {
-         xmlChar *match = NULL;
-         match = xmlGetProp (device, (const xmlChar *) "match");
+         xmlChar *match = xmlGetProp (device, (const xmlChar *) "match");
          if (match == NULL)
          {
             DEBUG(_("Warning: %s XML tag encountered with missing or bad attributes, ignored"), device->name);
@@ -422,8 +458,7 @@ int halevt_parse_config (char const *path)
                if (cur->type == XML_TEXT_NODE || cur->type == XML_COMMENT_NODE) { ; }
                else if (! xmlStrcmp (cur->name, (const xmlChar *) "Insertion"))
                {
-                  xmlChar *exec = NULL;
-                  exec =  xmlGetProp (cur, (const xmlChar *) "exec");
+                  xmlChar *exec = xmlGetProp (cur, (const xmlChar *) "exec");
                   if (exec == NULL)
                   {
                      DEBUG(_("Warning: %s XML tag encountered with missing or bad attributes, ignored"), cur->name);
@@ -431,12 +466,12 @@ int halevt_parse_config (char const *path)
                   else
                   {
                      halevt_add_insertion (match, exec);
+                     xmlFree(exec);
                   }
                }
                else if (! xmlStrcmp (cur->name, (const xmlChar *) "Removal"))
                {
-                  xmlChar *exec = NULL;
-                  exec =  xmlGetProp (cur, (const xmlChar *) "exec");
+                  xmlChar *exec = xmlGetProp (cur, (const xmlChar *) "exec");
                   if (exec == NULL)
                   {
                      DEBUG(_("Warning: %s XML tag encountered with missing or bad attributes, ignored"), cur->name);
@@ -444,12 +479,12 @@ int halevt_parse_config (char const *path)
                   else
                   {
                      halevt_add_removal (match, exec);
+                     xmlFree(exec);
                   }
                }
                else if (! xmlStrcmp (cur->name, (const xmlChar *) "OnInit"))
                {
-                  xmlChar *exec = NULL;
-                  exec =  xmlGetProp (cur, (const xmlChar *) "exec");
+                  xmlChar *exec = xmlGetProp (cur, (const xmlChar *) "exec");
                   if (exec == NULL)
                   {
                      DEBUG(_("Warning: %s XML tag encountered with missing or bad attributes, ignored"), cur->name);
@@ -457,16 +492,14 @@ int halevt_parse_config (char const *path)
                   else
                   {
                      halevt_add_oninit (match, exec);
+                     xmlFree(exec);
                   }
                }
                else if (! xmlStrcmp(cur->name, (const xmlChar *) "Condition"))
                {
-                  xmlChar *exec = NULL;
-                  xmlChar *name = NULL;
-                  xmlChar *value = NULL;
-                  exec =  xmlGetProp(cur, (const xmlChar *) "exec");
-                  name =  xmlGetProp(cur, (const xmlChar *) "name");
-                  value =  xmlGetProp(cur, (const xmlChar *) "value");
+                  xmlChar *exec =  xmlGetProp(cur, (const xmlChar *) "exec");
+                  xmlChar *name =  xmlGetProp(cur, (const xmlChar *) "name");
+                  xmlChar *value =  xmlGetProp(cur, (const xmlChar *) "value");
                   if (exec == NULL || name == NULL)
                   {
                      DEBUG(_("Warning: %s XML tag encountered with missing or bad attributes, ignored"), cur->name);
@@ -475,11 +508,13 @@ int halevt_parse_config (char const *path)
                   {
                      halevt_add_condition(match, exec, name, value);
                   }
+                  xmlFree(exec);
+                  xmlFree(name);
+                  xmlFree(value);
                }
                else if (! xmlStrcmp(cur->name, (const xmlChar *) "Property"))
                {
-                  xmlChar *name = NULL;
-                  name =  xmlGetProp(cur, (const xmlChar *) "name");
+                  xmlChar *name = xmlGetProp(cur, (const xmlChar *) "name");
                   if (name == NULL)
                   {
                      DEBUG(_("Warning: %s XML tag encountered with missing or bad attributes, ignored"), cur->name);
@@ -500,10 +535,8 @@ int halevt_parse_config (char const *path)
                            }
                            else
                            {
-                              xmlChar *value = NULL;
-                              xmlChar *exec = NULL;
-                              exec =  xmlGetProp(property_value_node, (const xmlChar *) "exec");
-                              value =  xmlGetProp(property_value_node, (const xmlChar *) "value");
+                              xmlChar *exec = xmlGetProp(property_value_node, (const xmlChar *) "exec");
+                              xmlChar *value = xmlGetProp(property_value_node, (const xmlChar *) "value");
                               if (exec == NULL || value == NULL)
                               {
                                  DEBUG(_("Warning: %s XML tag encountered with missing or bad attributes, ignored"), property_value_node->name);
@@ -512,11 +545,14 @@ int halevt_parse_config (char const *path)
                               {
                                  halevt_add_property_value(new_property, exec, value);
                               }
+                              xmlFree(exec);
+                              xmlFree(value);
                            }
                            property_value_node = property_value_node->next;
                         }
                      }
                   }
+                  xmlFree(name);
                }
                else
                {
@@ -525,6 +561,7 @@ int halevt_parse_config (char const *path)
                cur = cur->next;
             }
          }
+         xmlFree(match);
       }
       device = device->next;
    }
@@ -571,3 +608,13 @@ void halevt_print_config()
       property = property->next;
    }
 }
+
+void halevt_free_config()
+{
+   FREE_LINKED_LIST(halevt_property, halevt_property_root, halevt_free_property);
+   FREE_LINKED_LIST(halevt_insertion, halevt_insertion_root, halevt_free_insertion);
+   FREE_LINKED_LIST(halevt_removal, halevt_removal_root, halevt_free_removal);
+   FREE_LINKED_LIST(halevt_condition, halevt_condition_root, halevt_free_condition);
+   FREE_LINKED_LIST(halevt_oninit, halevt_oninit_root, halevt_free_oninit);
+   xmlCleanupParser();
+}
