@@ -31,7 +31,7 @@ typedef struct halevt_exec_element
 typedef struct halevt_exec
 {
    char *string;
-   char *parsed_string; 
+   char *parsed_string;
    halevt_exec_element *elements;
 } halevt_exec;
 
@@ -81,12 +81,13 @@ typedef struct halevt_condition
 } halevt_condition;
 
 int halevt_parse_config(char const *path);
+void halevt_free_config();
 char *halevt_hal_string (char *string);
 void halevt_print_config ();
 
-halevt_property *halevt_property_root;
-halevt_insertion *halevt_insertion_root;
-halevt_removal *halevt_removal_root;
-halevt_condition *halevt_condition_root;
-halevt_oninit *halevt_oninit_root;
+extern halevt_property *halevt_property_root;
+extern halevt_insertion *halevt_insertion_root;
+extern halevt_removal *halevt_removal_root;
+extern halevt_condition *halevt_condition_root;
+extern halevt_oninit *halevt_oninit_root;
 #endif
